﻿package de.davidochmann.frameworks.dosite.main.xmlparser 
{
	/**
	 * @author dochmann
	 */

	import de.davidochmann.frameworks.dosite.main.data.Data;
	import de.davidochmann.frameworks.dosite.main.data.composition.Composition;
	import de.davidochmann.frameworks.dosite.main.data.composition.item.Init;
	import de.davidochmann.frameworks.dosite.main.data.composition.item.Setup;
	import de.davidochmann.frameworks.dosite.main.data.composition.item.Transition;
	import de.davidochmann.frameworks.dosite.main.data.composition.item.layout.Layout;
	import de.davidochmann.frameworks.dosite.main.data.composition.item.layout.item.Fixed;
	import de.davidochmann.frameworks.dosite.main.data.composition.item.layout.item.Subsite;
	import de.davidochmann.frameworks.dosite.main.data.content.Content;
	import de.davidochmann.frameworks.dosite.main.data.content.item.Extensible;
	import de.davidochmann.frameworks.dosite.main.data.content.item.Graphic;
	
	public class XMLParser 
	{
		private var mContentXML:XML;
		private var mCompositionXML:XML;
		
		private var mData:Data;
		
		
		
		public function XMLParser(pContentXML:XML	  = null,
								  pCompositionXML:XML = null):void
		{
			setContentXML(pContentXML);
			setCompositionXML(pCompositionXML);
		}



		/*
		 * PUBLIC FUNCTIONS
		 */
		
		public function init():void
		{
			initData();
			initDefaults();
			initContent();
			initComposition();
		}
		
		
		
		/*
		 * GETTER FUNCTIONS
		 */
		
		public function getContentXML():XML
		{
			return mContentXML;
		}

		public function getCompositionXML():XML
		{
			return mCompositionXML;
		}


		
		/*
		 * SETTER FUNCTIONS
		 */
		
		public function setContentXML(pContentXML:XML):void
		{
			mContentXML = pContentXML;
		}
		
		public function setCompositionXML(pCompositionXML:XML):void
		{
			mCompositionXML = pCompositionXML;
		}
		
		
		
		/*
		 * PRIVATE FUNCTIONS
		 */
		
		private function initData():void
		{
			mData = Data.getInstance();
		}

		private function initDefaults():void
		{
			var cmp:Composition = mData.composition();
			var stp:Setup = new Setup();
			
			stp.setPosition("tl");
			
			
			var ini:Init = new Init();
			
			ini.setURL("");
			ini.setParams("");
			
			
			var trs:Transition = new Transition();

			trs.setSwap(0);
			trs.setWait(false);
			
			
			var lay:Layout = new Layout();
			
			lay.setFixedIndex(0);
			lay.setSubsiteIndex(1);
					
					
			cmp.addSetup(stp);
			cmp.addInit(ini);
			cmp.addTransition(trs);
			cmp.addLayout(lay);
		}

		
		
		/*
		 * CONTENT FUNCTION
		 */

		private function initContent():void
		{	
			if(mContentXML == null) return;
			
			var cnt:Content = mData.content();
			
			
			var extensible:Extensible;
			var elength:uint = XMLList(mContentXML.CONTENT.EXTENSIBLE.ITEM).length();
			
			for(var i:uint = 0; i < elength; ++i)
			{
				extensible = new Extensible();
				extensible.setId(String(XMLList(mContentXML.CONTENT.EXTENSIBLE.ITEM[i]).attribute("id")));
				extensible.setPath(String(XMLList(mContentXML.CONTENT.EXTENSIBLE.ITEM[i]).attribute("path")));
				
				cnt.addExtensible(extensible);
			}
			
			
			var graphic:Graphic;
			var gLenght:uint = mContentXML.CONTENT.GRAPHIC.ITEM.length();
			
			for(var j:uint = 0; j < gLenght; ++j)
			{
				graphic = new Graphic();
				graphic.setId(String(XMLList(mContentXML.CONTENT.GRAPHIC.ITEM[j]).attribute("id")));
				graphic.setPath(String(XMLList(mContentXML.CONTENT.GRAPHIC.ITEM[j]).attribute("path")));
								
				cnt.addGraphic(graphic);
			}
		}



		/*
		 * COMPOSITION FUNCTION
		 */

		private function initComposition():void
		{
			if(mCompositionXML == null) return;
			
			var cmp:Composition = mData.composition();
			
						
			var setup:Setup = new Setup();
			
			setup.setPosition(String(XMLList(mCompositionXML.COMPOSITION.SETUP).attribute("position")));

			cmp.addSetup(setup);

			
			var init:Init = new Init();
			
			init.setURL(String(XMLList(mCompositionXML.COMPOSITION.INIT).attribute("url")));
			init.setParams(String(XMLList(mCompositionXML.COMPOSITION.INIT).attribute("params")));

			cmp.addInit(init);


			var transition:Transition = new Transition();
			
			transition.setSwap(uint(XMLList(mCompositionXML.COMPOSITION.TRANSITION).attribute("swap")));
			transition.setWait(Boolean((String(XMLList(mCompositionXML.COMPOSITION.TRANSITION).attribute("wait"))) == "false") ? false : true);
			
			cmp.addTransition(transition);
												
						
			var layout:Layout = new Layout();
			
			layout.setSubsiteIndex(uint(XMLList(mCompositionXML.COMPOSITION.LAYOUT.SUBSITE).childIndex())); 
			layout.setFixedIndex(uint(XMLList(mCompositionXML.COMPOSITION.LAYOUT.FIXED).childIndex()));
			
			var subsite:Subsite;
			var slength:uint = XMLList(mCompositionXML.COMPOSITION.LAYOUT.SUBSITE.ITEM).length();
			
			for(var i:uint = 0; i < slength; ++i)
			{
				subsite = new Subsite();
				
				subsite.setId(String(XMLList(mCompositionXML.COMPOSITION.LAYOUT.SUBSITE.ITEM[i]).attribute("id")));
				subsite.setURL(String(XMLList(mCompositionXML.COMPOSITION.LAYOUT.SUBSITE.ITEM[i]).attribute("url")));
				subsite.setParams(String(XMLList(mCompositionXML.COMPOSITION.LAYOUT.SUBSITE.ITEM[i]).attribute("params")));
				subsite.setTitle(String(XMLList(mCompositionXML.COMPOSITION.LAYOUT.SUBSITE.ITEM[i]).attribute("title")));
				subsite.setWidth(uint(XMLList(mCompositionXML.COMPOSITION.LAYOUT.SUBSITE.ITEM[i]).attribute("width")));
				subsite.setHeight(uint(XMLList(mCompositionXML.COMPOSITION.LAYOUT.SUBSITE.ITEM[i]).attribute("height")));
				subsite.setX(int(XMLList(mCompositionXML.COMPOSITION.LAYOUT.SUBSITE.ITEM[i]).attribute("x")));
				subsite.setY(int(XMLList(mCompositionXML.COMPOSITION.LAYOUT.SUBSITE.ITEM[i]).attribute("y")));
				
				layout.addSubsite(subsite);				
			}
			
			var fixed:Fixed;
			var clength:uint = XMLList(mCompositionXML.COMPOSITION.LAYOUT.FIXED.ITEM).length();
			
			for(var j:uint = 0; j < clength; ++j)
			{
				fixed = new Fixed();
				
				fixed.setId(String(XMLList(mCompositionXML.COMPOSITION.LAYOUT.FIXED.ITEM[j]).attribute("id")));
				fixed.setTitle(String(XMLList(mCompositionXML.COMPOSITION.LAYOUT.FIXED.ITEM[j]).attribute("title")));
				fixed.setWidth(uint(XMLList(mCompositionXML.COMPOSITION.LAYOUT.FIXED.ITEM[j]).attribute("width")));
				fixed.setHeight(uint(XMLList(mCompositionXML.COMPOSITION.LAYOUT.FIXED.ITEM[j]).attribute("height")));
				fixed.setX(int(XMLList(mCompositionXML.COMPOSITION.LAYOUT.FIXED.ITEM[j]).attribute("x")));
				fixed.setY(int(XMLList(mCompositionXML.COMPOSITION.LAYOUT.FIXED.ITEM[j]).attribute("y")));
				
				layout.addFixed(fixed);
			}
			
			cmp.addLayout(layout);
		}
	}
}
