﻿package de.davidochmann.frameworks.dosite.main.containerbuilder 
{
	/**
	 * @author David Ochmann
	 */
	import de.davidochmann.frameworks.dosite.main.containerbuilder.containerfactory.ContainerFactory;
	import de.davidochmann.frameworks.dosite.main.containerbuilder.containerfactory.container.FixedContainer;
	import de.davidochmann.frameworks.dosite.main.containerbuilder.containerfactory.container.SubsiteContainer;
	import de.davidochmann.frameworks.dosite.main.data.Data;
	import de.davidochmann.frameworks.dosite.main.data.composition.Composition;
	import de.davidochmann.frameworks.dosite.main.data.composition.item.layout.item.Fixed;
	import de.davidochmann.frameworks.dosite.main.data.composition.item.layout.item.Subsite;
	import de.davidochmann.frameworks.dosite.main.data.content.Content;
	import de.davidochmann.frameworks.dosite.main.data.content.item.Extensible;
	import de.davidochmann.frameworks.dosite.main.data.content.item.Graphic;
	import de.davidochmann.frameworks.dosite.main.displayregister.DisplayRegister;

	import flash.display.DisplayObjectContainer;
	public class ContainerBuilder 
	{
		private var mData:Data;
		private var mDisplayRegister:DisplayRegister;		
		private var mContainerFactory:ContainerFactory;
		private var mContainerList:Array;
		
		
		
		public function ContainerBuilder(pDisplayRegister:DisplayRegister):void
		{
			setDisplayRegister(pDisplayRegister);
		}
		
		
		
		/*
		 * PUBLIC FUNCTIONS 
		 */
		
		public function init():void
		{
			initData();
			initContainerFactory();
			initContainerList();
			initExtensibleContainer();				
		}
			
		public function containerList():Array
		{
			return mContainerList;
		}
		
		public function containerFactory():ContainerFactory
		{
			return mContainerFactory;
		}

		
		
		/*
		 * GETTER FUNCTIONS
		 */

		public function getDisplayRegister():DisplayRegister
		{
			return mDisplayRegister;
		}


		
		/*
		 * SETTER FUNCTIONS
		 */
		 
		public function setDisplayRegister(pDisplayRegister:DisplayRegister):void
		{
			mDisplayRegister = pDisplayRegister;
		}

		
		
		/*
		 * PRIVATE FUNCTIONS
		 */
		
		private function initData():void
		{
			mData = Data.getInstance();
		}

		private function initContainerFactory():void
		{
			mContainerFactory = new ContainerFactory();
		}
		
		private function initContainerList():void
		{
			mContainerList = new Array();
		}
		
		private function initExtensibleContainer():void
		{
			var com:Composition = mData.composition();
			var con:Content		= mData.content(); 

			var subsList:Array;
			var contList:Array;			
			var clength:uint;
			var slength:uint;
			
			var extensible:Extensible;
			var graphic:Graphic; 
			
			
			var fixed:Fixed;
			var fixC:FixedContainer;
			
			var fId:String					  =   "";
			var fTitle:String				  =   "";
			var fPath:String				  =   "";
			var fExtensible:String			  =   "";
			var fChild:DisplayObjectContainer = null;
			var fX:int						  =    0;
			var fY:int						  =    0;
			var fWidth:uint					  =    0;
			var fHeight:uint				  =    0; 
			var fDepth:uint					  =    0;

			
			var subsite:Subsite;
			var subC:SubsiteContainer;

			var sId:String					  =   "";
			var sURL:String					  =   "";
			var sParams:String				  =   "";
			var sTitle:String				  =   "";
			var sPath:String				  =   "";
			var sExtensible:String			  =   "";
			var sChild:DisplayObjectContainer = null;
			var sX:int						  =    0;
			var sY:int						  =    0;
			var sWidth:uint					  =    0;
			var sHeight:uint				  =    0; 
			var sDepth:uint					  =    0;
	

			if(com != null && con != null && com.layout() != null)
			{
				subsList = com.layout().subsiteList();
				contList = com.layout().fixedList();
				
				clength	 = contList.length;
				slength	 = subsList.length;


				for(var i:uint = 0;i < clength; ++i)
				{
					fixed  		= Fixed(contList[i]);
						
					fId 		= fixed.getId();
		
					extensible  = con.extensible(fId);
					graphic		= con.graphic(fId);
					
					fTitle  	= fixed.getTitle();
					fPath		= graphic.getPath();
					fChild		= graphic.getChild();
					fX	   		= fixed.getX();
					fY	  		= fixed.getY();
					fWidth		= fixed.getWidth();
					fHeight		= fixed.getHeight();
					fDepth  	= i;
		
					if(extensible != null) fExtensible = extensible.getPath();
					
					fixC = ContainerFactory.fixed(fId, fTitle, fPath, fExtensible, fChild, fX, fY, fWidth, fHeight, fDepth);
					mDisplayRegister.addContainer(fixC);
				}
			
			
				for(var j:uint = 0;j < slength; ++j)
				{
					subsite = Subsite(subsList[j]);
	
					sId 		= subsite.getId();
						
					extensible  = con.extensible(sId);
					graphic		= con.graphic(sId);
					
					sURL   		= subsite.getURL();
					sParams		= subsite.getParams();		
					sTitle  	= subsite.getTitle();
					sPath		= graphic.getPath();
					sChild		= graphic.getChild();
					sX	   		= subsite.getX();
					sY	  		= subsite.getY();
					sWidth		= subsite.getWidth();
					sHeight		= subsite.getHeight();
					sDepth  	= j;
					
					
					if(extensible != null) sExtensible = extensible.getPath();
					
					subC = ContainerFactory.subsite(sId, sURL, sParams, sTitle, sPath, sExtensible, sChild, sX, sY, sWidth, sHeight);
					mDisplayRegister.addSubsite(subC);		
				}
			}
		}
	}
}
