﻿package de.davidochmann.frameworks.dosite.main.navigation 
{
	/**
	 * @author dochmann
	 */
	 
	import de.davidochmann.frameworks.dosite.main.containerbuilder.containerfactory.container.SubsiteContainer;
	import de.davidochmann.frameworks.dosite.main.data.Data;
	import de.davidochmann.frameworks.dosite.main.display.Display;
	import de.davidochmann.frameworks.dosite.main.displayregister.DisplayRegister;
	import de.davidochmann.frameworks.dosite.main.events.DoSiteEvent;

	import flash.display.Sprite;
	import flash.events.EventDispatcher;
	import flash.utils.setTimeout;
	
	public class Navigation extends EventDispatcher
	{
		private var mData:Data;
		private var mDisplayRegister:DisplayRegister;
		private var mDisplay:Display;

		private var mCurrent:SubsiteContainer;		
		private var mTransition:Boolean;		
		private var mWait:Boolean;


		
		public function Navigation(pDisplayRegister:DisplayRegister,
								   pDisplay:Display):void
		{
			setDisplayRegister(pDisplayRegister);
			setDisplay(pDisplay);
			initData();
		}

		
		
		/*
		 * PUBLIC FUNCTIONS
		 */
		
		public function init():void
		{
			initTransition();
			initWait();
			openInitialSubsite();
		}
		
		public function open(pURL:String):SubsiteContainer
		{	
			var spl:Array  = pURL.split("?");
			var url:String = String(spl[0]);
			var pms:String = (spl[1] != undefined && spl[1] != null) ? "?" + String(spl[1]) : "";

			if(mTransition && mWait) return null;

			mTransition = false;
			
			var container:Sprite  = mDisplay.subsiteDisplay();
			
			var subsiteList:Array = mDisplayRegister.subsiteContainerList();
			var sLength:uint	  = subsiteList.length;
			
			var subsite:SubsiteContainer;
						
			for(var i:uint = 0; i < sLength; ++i)
			{
				subsite = SubsiteContainer(subsiteList[i]);
				if(subsite.getURL() == url) break;
				subsite = null;
			}
		
			if(subsite != null) 
			{
				while(container.numChildren > 0) container.removeChildAt(0);
	
				mCurrent = subsite;
				
				container.addChild(subsite);
				subsite.setParams(pms);
				
				dispatchEvent(new DoSiteEvent(DoSiteEvent.NAVIGATION_SITE_OPEN, subsite));
			}
			else if(pURL == "")
			{
				while(container.numChildren > 0) container.removeChildAt(0);
				
				mCurrent = null;
				dispatchEvent(new DoSiteEvent(DoSiteEvent.NAVIGATION_SITE_OPEN, subsite));
			}
			
			return subsite;
		}


		public function swap(pURL:String):void
		{
			var url:String = (mCurrent != null) ? mCurrent.getURL() : "";
			if((mTransition && mWait) || (pURL == url && pURL != "")) return;
			
			mTransition = true;

			var swap:uint = mData.composition().transition().getSwap();
			dispatchEvent(new DoSiteEvent(DoSiteEvent.NAVIGATION_SITE_SWAP_INIT, current()));
						
			setTimeout(swapOpen, swap, pURL);
		}

		private function swapOpen(pURL:String):SubsiteContainer
		{	
			mTransition = false;
			
			var container:Sprite  = mDisplay.subsiteDisplay();
			
			var subsiteList:Array = mDisplayRegister.subsiteContainerList();
			var sLength:uint	  = subsiteList.length;
			
			var subsite:SubsiteContainer;
						
			for(var i:uint = 0; i < sLength; ++i)
			{
				subsite = SubsiteContainer(subsiteList[i]);
				if(subsite.getURL() == pURL) break;
				subsite = null;
			}
		
			if(subsite != null) 
			{
				while(container.numChildren > 0) container.removeChildAt(0);
	
				mCurrent = subsite;
				
				container.addChild(subsite);
				dispatchEvent(new DoSiteEvent(DoSiteEvent.NAVIGATION_SITE_SWAP_OPEN, subsite));
			}
			else if(pURL == "")
			{
				while(container.numChildren > 0) container.removeChildAt(0);
				
				mCurrent = null;
				dispatchEvent(new DoSiteEvent(DoSiteEvent.NAVIGATION_SITE_SWAP_OPEN, subsite));
			}
			
			
			return subsite;
		}

		public function transition():Boolean
		{
			return mTransition;
		}

		public function current():SubsiteContainer
		{
			return mCurrent;
		}
		
		
		
		/*
		 * GETTER FUNCTIONS
		 */

		public function getDisplayRegister():DisplayRegister
		{
			return mDisplayRegister;
		}

		public function getDisplay():Display
		{
			return mDisplay;
		}


		
		/*
		 * SETTER FUNCTIONS
		 */
		 
		public function setDisplayRegister(pDisplayRegister:DisplayRegister):void
		{
			mDisplayRegister = pDisplayRegister;
		}

		public function setDisplay(pDisplay:Display):void
		{
			mDisplay = pDisplay;		
		}

		
		
		/*
		 * PRIVATE FUNCTIONS
		 */
		
		private function initData():void
		{
			mData = Data.getInstance();
		}
		
		private function initTransition():void
		{
			mTransition = false;
		}

		private function initWait():void
		{
			mWait = mData.composition().transition().getWait();
		}

		private function openInitialSubsite():void
		{	
			var init:String = mData.composition().init().getURL();
			if(init != "" && init != null) open(init);
		}
	}
}