﻿package de.davidochmann.frameworks.dosite 
{
	/**
	 * DoSite
	 * @version v0.61
	 * @author 	David Ochmann
	 * 
	 * @mail	mail@davidochmann.de 	
	 * @website www.davidochmann.de
	 * 
	 * 
	 * v0.61
	 * - Added a Data class that holds every framework property. It replaces the content and composition classes in 
	 *   the “xmlparser” package.
	 * - It is no longer necessary to assign the content or composition xml to the framework. All properties can now 
	 *   be set via the new Data class or directly with DoSite.compositon() or DoSite.content() after initializing DoSite.
	 * - DoSite can now be initialized without an open SubSite. This is important for utilizing a See-and-Point 
	 *   navigation structure.
	 * - Removed a bug that caused the content to align to the wrong position when the container align is set to „tl“.
	 * - Renamed the addSubsite function to addSubsiteContainer.
	 * - Renamed the addFixed function to addFixedContainer.
	 * - Various code optimizations.
	 *
	 */
	 
	import de.davidochmann.frameworks.dosite.main.Main;
	import de.davidochmann.frameworks.dosite.main.containerbuilder.ContainerBuilder;
	import de.davidochmann.frameworks.dosite.main.containerbuilder.containerfactory.container.FixedContainer;
	import de.davidochmann.frameworks.dosite.main.containerbuilder.containerfactory.container.SubsiteContainer;
	import de.davidochmann.frameworks.dosite.main.data.Data;
	import de.davidochmann.frameworks.dosite.main.data.composition.Composition;
	import de.davidochmann.frameworks.dosite.main.data.content.Content;
	import de.davidochmann.frameworks.dosite.main.display.Display;
	import de.davidochmann.frameworks.dosite.main.displayregister.DisplayRegister;
	import de.davidochmann.frameworks.dosite.main.events.DoSiteEvent;
	import de.davidochmann.frameworks.dosite.main.navigation.Navigation;
	import de.davidochmann.frameworks.dosite.main.xmlparser.XMLParser;

	import flash.display.Sprite;
	import flash.events.EventDispatcher;
	import flash.utils.getDefinitionByName;
	
	public class DoSite extends EventDispatcher
	{
		private static var mInstance:DoSite;

		private var mContentXML:XML;
		private var mCompositionXML:XML;
		
		private var mMain:Main;



		/*
		 * SINGLETON CONSTUCTOR
		 */
		
		public static function getInstance():DoSite
		{
			if(mInstance == null) mInstance = new DoSite(new SingletonEnforcer());				
			return mInstance;
		}
		
		public function DoSite(pSingletonEnforcer:SingletonEnforcer):void{}
		
		public function initialize(pContentXML:XML	   = null,
								   pCompositionXML:XML = null):void
		{
			setContentXML(pContentXML);
			setCompositionXML(pCompositionXML);
			
			initMain();
			initEventHandler();
		}
		
		public function start():void
		{
			mMain.start();	
		}

		
		
		/*
		 * PUBLIC FUNCTIONS
		 */

		public function addSubsiteContainer(pSubsiteContainer:SubsiteContainer):void
		{
			mMain.displayRegister().addSubsite(pSubsiteContainer);
			mMain.contentInitializer().initSubsite(pSubsiteContainer);
		}
		
		public function addFixedContainer(pFixedContainer:FixedContainer):void
		{
			mMain.displayRegister().addContainer(pFixedContainer);
			mMain.contentInitializer().initContainer(pFixedContainer);
			mMain.display().fixedDisplay().addChild(new Sprite());
		}
		 
		public function main():Main
		{
			return mMain; 
		}
		
		public function data():Data
		{
			return mMain.data();
		}

		public function xmlParser():XMLParser
		{
			return mMain.xmlParser();
		}
		
		public function content():Content
		{
			return mMain.data().content();
		}
		
		public function composition():Composition
		{
			return mMain.data().composition();
		}

		public function displayRegister():DisplayRegister
		{
			return mMain.displayRegister();
		}

		public function subsiteContainer(pId:String):SubsiteContainer
		{
			return mMain.displayRegister().subsiteContainer(pId);
		}
		
		public function fixedContainer(pId:String):FixedContainer
		{
			return mMain.displayRegister().fixedContainer(pId);
		}

		public function containerBuilder():ContainerBuilder
		{
			return mMain.containerBuilder();
		}

		public function display():Display
		{
			return mMain.display();
		}
		
		public function container():Sprite
		{
			return mMain.display().container();
		}

		public function navigation():Navigation
		{
			return mMain.navigation();
		}

		public function current():SubsiteContainer
		{
			return mMain.navigation().current();
		}

		public function transition():Boolean
		{
			return mMain.navigation().transition();
		}
		
		public function open(pURL:String):SubsiteContainer
		{
			return mMain.navigation().open(pURL);
		}
		
		public function swap(pURL:String):void
		{
			mMain.navigation().swap(pURL);
		}

		
		
		/*
		 * GETTER FUNCTIONS
		 */
		
		public function getContentXML():XML
		{
			return mContentXML;
		}

		public function getCompositionXML():XML
		{
			return mCompositionXML;
		}


		
		/*
		 * SETTER FUNCTIONS
		 */
		
		public function setContentXML(pContentXML:XML):void
		{
			mContentXML = pContentXML;
		}

		public function setCompositionXML(pCompositionXML:XML):void
		{
			mCompositionXML = pCompositionXML;
		}
		
		
		
		/*
		 * INIT MAIN
		 */

		private function initMain():void
		{
			mMain = (getDefinitionByName("de.davidochmann.frameworks.dosite.main.Main") as Class).getInstance();
			mMain.init(mContentXML, mCompositionXML);
		}
		
		private function initEventHandler():void
		{
			mMain.addEventListener(DoSiteEvent.NAVIGATION_SITE_OPEN, doSiteEventHandler);
			mMain.addEventListener(DoSiteEvent.NAVIGATION_SITE_SWAP_INIT, doSiteEventHandler);
			mMain.addEventListener(DoSiteEvent.NAVIGATION_SITE_SWAP_OPEN, doSiteEventHandler);
		}
		
		private function doSiteEventHandler(e:DoSiteEvent):void
		{
			dispatchEvent(e);
		}
	}
}

class SingletonEnforcer{}