﻿package de.davidochmann.frameworks.dosite.main
{
	/**
	 * @author dochmann
	 */
	 
	import de.davidochmann.frameworks.dosite.main.containerbuilder.ContainerBuilder;
	import de.davidochmann.frameworks.dosite.main.contentinitializer.ContentInitializer;
	import de.davidochmann.frameworks.dosite.main.data.Data;
	import de.davidochmann.frameworks.dosite.main.display.Display;
	import de.davidochmann.frameworks.dosite.main.displayregister.DisplayRegister;
	import de.davidochmann.frameworks.dosite.main.events.DoSiteEvent;
	import de.davidochmann.frameworks.dosite.main.navigation.Navigation;
	import de.davidochmann.frameworks.dosite.main.xmlparser.XMLParser;

	import flash.events.EventDispatcher;
	
	public class Main extends EventDispatcher
	{
		private static var mInstance:Main;
		
		private var mContentXML:XML;
		private var mCompositionXML:XML;
		
		private var mData:Data;
		private var mXMLParser:XMLParser;
		private var mDisplayRegister:DisplayRegister;
		private var mContainerBuilder:ContainerBuilder;
		private var mContentInitializer:ContentInitializer;
		private var mDisplay:Display;
		private var mNavigation:Navigation;
		

		
		/*
		 * SINGLETON CONSTRUCTOR
		 */
		
		public static function getInstance():Main
		{
			if(mInstance == null) mInstance = new Main(new SingletonEnforcer());
			return mInstance;
		}

		public function Main(pSingletonEnforcer:SingletonEnforcer):void{}
		
		public function init(pContentXML:XML	 = null, 
							 pCompositionXML:XML = null):void
		{
			setContentXML(pContentXML);
			setCompositionXML(pCompositionXML);
			
			initData();
			initXMLParser();
			initDisplayRegister();
			initContainerBuilder();
			initContentInitializer();
			initDisplay();
			initNavigation();
			initEventHandler();
		}

		public function start():void
		{		
			mContainerBuilder.init();
			mContentInitializer.init();
			mDisplay.init();
			mNavigation.init();
		}

		
		
		/*
		 * PUBLIC FUNCTIONS
		 */
		
		public function data():Data
		{
			return mData;
		}

		public function xmlParser():XMLParser
		{
			return mXMLParser;
		}

		public function displayRegister():DisplayRegister
		{
			return mDisplayRegister;
		}

		public function containerBuilder():ContainerBuilder
		{
			return mContainerBuilder;
		}

		public function contentInitializer():ContentInitializer
		{
			return mContentInitializer;
		}

		public function display():Display
		{
			return mDisplay;
		}

		public function navigation():Navigation
		{
			return mNavigation;
		}

		
		
		/*
		 * GETTER FUNCTIONS
		 */
		
		public function getContentXML():XML
		{
			return mContentXML;
		}

		public function getCompositionXML():XML
		{
			return mCompositionXML;
		}


		
		/*
		 * SETTER FUNCTIONS
		 */
		
		public function setContentXML(pContentXML:XML):void
		{
			mContentXML = pContentXML;
		}

		public function setCompositionXML(pCompositionXML:XML):void
		{
			mCompositionXML = pCompositionXML;
		}


		
		/*
		 * PRIVATE FUNCTIONS
		 */
		
		private function initData():void
		{
			mData = Data.getInstance();
			mData.init();
		}

		private function initXMLParser():void
		{
			mXMLParser = new XMLParser(mContentXML, mCompositionXML);
			mXMLParser.init();
		}
		
		private function initDisplayRegister():void
		{
			mDisplayRegister = new DisplayRegister();
			mDisplayRegister.init();			
		}
		
		private function initContainerBuilder():void
		{
			mContainerBuilder = new ContainerBuilder(mDisplayRegister);
		}

		private function initContentInitializer():void
		{
			mContentInitializer = new ContentInitializer(mDisplayRegister); 
		}
		
		private function initDisplay():void
		{
			mDisplay = new Display(mDisplayRegister, mXMLParser);
			mDisplay.init();
		}
		
		private function initNavigation():void
		{
			mNavigation = new Navigation(mDisplayRegister, mDisplay);
		}
		
		private function initEventHandler():void
		{
			mNavigation.addEventListener(DoSiteEvent.NAVIGATION_SITE_OPEN, doSiteEventHandler);
			mNavigation.addEventListener(DoSiteEvent.NAVIGATION_SITE_SWAP_INIT, doSiteEventHandler);
			mNavigation.addEventListener(DoSiteEvent.NAVIGATION_SITE_SWAP_OPEN, doSiteEventHandler);
		}
		
		private function doSiteEventHandler(e:DoSiteEvent):void
		{
			dispatchEvent(e);
		}
	}
}

class SingletonEnforcer{}
