﻿
package de.davidochmann.frameworks.dosite 
{
	/**
	 * @author dochmann
	 */
	 
	import flash.display.Sprite;
	import flash.events.EventDispatcher;
	import flash.utils.getDefinitionByName;
	import de.davidochmann.frameworks.dosite.main.Main;
	import de.davidochmann.frameworks.dosite.main.display.Display;
	import de.davidochmann.frameworks.dosite.main.events.DoSiteEvent;
	import de.davidochmann.frameworks.dosite.main.xmlparser.XMLParser;
	import de.davidochmann.frameworks.dosite.main.navigation.Navigation;
	import de.davidochmann.frameworks.dosite.main.xmlparser.data.content.Content;
	import de.davidochmann.frameworks.dosite.main.displayregister.DisplayRegister;
	import de.davidochmann.frameworks.dosite.main.containerbuilder.ContainerBuilder;
	import de.davidochmann.frameworks.dosite.main.xmlparser.data.composition.Composition;
	import de.davidochmann.frameworks.dosite.main.containerbuilder.containerfactory.container.SubsiteContainer;
	import de.davidochmann.frameworks.dosite.main.containerbuilder.containerfactory.container.FixedContainer;
	
	public class DoSite extends EventDispatcher
	{
		private static var mInstance:DoSite;

		private var mContentXML:XML;
		private var mCompositionXML:XML;
		
		private var mMain:Main;



		/*
		 * SINGLETON CONSTUCTOR
		 */
		
		public static function getInstance():DoSite
		{
			if(mInstance == null) mInstance = new DoSite(new SingletonEnforcer());				
			return mInstance;
		}
		
		public function DoSite(pSingletonEnforcer:SingletonEnforcer):void{}
		
		public function initialize(pContentXML:XML	   = null,
								   pCompositionXML:XML = null):void
		{
			setContentXML(pContentXML);
			setCompositionXML(pCompositionXML);
			
			initMain();
			initEventHandler();
		}
		
		public function start():void
		{
			mMain.start();	
		}

		
		
		/*
		 * PUBLIC FUNCTIONS
		 */

		public function addSubsite(pSubsiteContainer:SubsiteContainer):void
		{
			mMain.displayRegister().addSubsite(pSubsiteContainer);
			mMain.contentInitializer().initSubsite(pSubsiteContainer);
		}
		
		public function addFixed(pFixedContainer:FixedContainer):void
		{
			mMain.displayRegister().addContainer(pFixedContainer);
			mMain.contentInitializer().initContainer(pFixedContainer);
			mMain.display().fixedDisplay().addChild(new Sprite());
		}
		 
		public function main():Main
		{
			return mMain; 
		}
		
		public function xmlParser():XMLParser
		{
			return mMain.xmlParser();
		}
		
		public function content():Content
		{
			return mMain.xmlParser().content();
		}
		
		public function composition():Composition
		{
			return mMain.xmlParser().composition();
		}

		public function displayRegister():DisplayRegister
		{
			return mMain.displayRegister();
		}

		public function subsiteContainer(pId:String):SubsiteContainer
		{
			return mMain.displayRegister().subsiteContainer(pId);
		}
		
		public function fixedContainer(pId:String):FixedContainer
		{
			return mMain.displayRegister().fixedContainer(pId);
		}

		public function containerBuilder():ContainerBuilder
		{
			return mMain.containerBuilder();
		}

		public function display():Display
		{
			return mMain.display();
		}
		
		public function container():Sprite
		{
			return mMain.display().container();
		}

		public function navigation():Navigation
		{
			return mMain.navigation();
		}

		public function current():SubsiteContainer
		{
			return mMain.navigation().current();
		}

		public function transition():Boolean
		{
			return mMain.navigation().transition();
		}
		
		public function open(pURL:String):SubsiteContainer
		{
			return mMain.navigation().open(pURL);
		}
		
		public function swap(pURL:String):void
		{
			mMain.navigation().swap(pURL);
		}

		
		
		/*
		 * GETTER FUNCTIONS
		 */
		
		public function getContentXML():XML
		{
			return mContentXML;
		}

		public function getCompositionXML():XML
		{
			return mCompositionXML;
		}


		
		/*
		 * SETTER FUNCTIONS
		 */
		
		public function setContentXML(pContentXML:XML):void
		{
			mContentXML = pContentXML;
		}

		public function setCompositionXML(pCompositionXML:XML):void
		{
			mCompositionXML = pCompositionXML;
		}
		
		
		
		/*
		 * INIT MAIN
		 */

		private function initMain():void
		{
			mMain = (getDefinitionByName("de.davidochmann.frameworks.dosite.main.Main") as Class).getInstance();
			mMain.initialize(mContentXML, mCompositionXML);
		}
		
		private function initEventHandler():void
		{
			mMain.addEventListener(DoSiteEvent.NAVIGATION_SITE_OPEN, doSiteEventHandler);
			mMain.addEventListener(DoSiteEvent.NAVIGATION_SITE_SWAP_INIT, doSiteEventHandler);
			mMain.addEventListener(DoSiteEvent.NAVIGATION_SITE_SWAP_OPEN, doSiteEventHandler);
		}
		
		private function doSiteEventHandler(e:DoSiteEvent):void
		{
			dispatchEvent(e);
		}
	}
}

class SingletonEnforcer{}