﻿package de.davidochmann.frameworks.dosite.main.display 
{
	/**
	 * @author dochmann
	 */

	import flash.events.Event;
	import flash.display.Stage;
	import flash.display.Sprite;
	import de.davidochmann.frameworks.dosite.main.xmlparser.XMLParser;
	import de.davidochmann.frameworks.dosite.main.displayregister.DisplayRegister;
	import de.davidochmann.frameworks.dosite.main.containerbuilder.containerfactory.container.FixedContainer;
	 
	public class Display 
	{
		private var mContainer:Sprite;
		private var mSubsiteDisplay:Sprite;
		private var mFixedDisplay:Sprite;
		
		private var mDisplayRegister:DisplayRegister;
		private var mXMLParser:XMLParser;
		
		
		
		public function Display(pDisplayRegister:DisplayRegister,
								pXMLParser:XMLParser):void
		{
			setDisplayRegister(pDisplayRegister);
			setXMLParser(pXMLParser);
			initSubsiteDisplay();
			initFixedDisplay();
			initContainer();
		}



		/*
		 * PUBLIC FUNCTIONS
		 */

		public function initialize():void
		{
			initContainerChildren();
		}

		public function container():Sprite
		{
			return mContainer;
		}

		public function fixedDisplay():Sprite
		{
			return mFixedDisplay;
		}

		public function subsiteDisplay():Sprite
		{
			return mSubsiteDisplay;
		}

		
		
		/*
		 * GETTER FUNCTIONS
		 */

		public function getDisplayRegister():DisplayRegister
		{
			return mDisplayRegister;	
		}

		public function getXMLParser():XMLParser
		{
			return mXMLParser;
		}

		
		
		/*
		 * SETTER FUNCTIONS
		 */

		public function setDisplayRegister(pDisplayRegister:DisplayRegister):void
		{
			mDisplayRegister = pDisplayRegister;
		}

		public function setXMLParser(pXMLParser:XMLParser):void
		{
			mXMLParser = pXMLParser;
		}

		
		
		/*
		 * PRIVATE FUNCTIONS
		 */		
		
		private function initSubsiteDisplay():void
		{
			mSubsiteDisplay = new Sprite();
		}

		private function initFixedDisplay():void
		{
			mFixedDisplay = new Sprite();
		}

		private function initContainer():void
		{
			mContainer	  = new Sprite();
			
			var c1:Sprite = new Sprite();
			var c2:Sprite = new Sprite();
			var fdi:uint  = uint(XMLList(mXMLParser.getCompositionXML().COMPOSITION.LAYOUT.SUBSITE).childIndex()); 
			var sdi:uint  = uint(XMLList(mXMLParser.getCompositionXML().COMPOSITION.LAYOUT.FIXED).childIndex());

			mContainer.addChild(c1);
			mContainer.addChild(c2);			

			mContainer.addChildAt(mFixedDisplay  , fdi);
			mContainer.addChildAt(mSubsiteDisplay, sdi);
			
			mContainer.removeChild(c1);
			mContainer.removeChild(c2);		
			
			c1 = null;
			c2 = null;
			
			mContainer.addEventListener(Event.ADDED_TO_STAGE, containerAddedToStageHandler);
		}
		

		private function containerAddedToStageHandler(e:Event):void	
		{
			var stage:Stage	    = mContainer.stage;
			var position:String = mXMLParser.composition().setup().getPosition(); 
						
			switch(position)
			{
				case "tl": 
					stage.addEventListener(Event.RESIZE, stageResizeHandlerTl);
					stageResizeHandlerTc(null);
					break;
				
				case "tc": 
					stage.addEventListener(Event.RESIZE, stageResizeHandlerTc);
					stageResizeHandlerTc(null);
					break;
					
				case "tr": 
					stage.addEventListener(Event.RESIZE, stageResizeHandlerTr);
					stageResizeHandlerTr(null);
					break;
					
				case "cl": 
					stage.addEventListener(Event.RESIZE, stageResizeHandlerCl);
					stageResizeHandlerCl(null);
					break;
				
				case "cc": 
					stage.addEventListener(Event.RESIZE, stageResizeHandlerCc);
					stageResizeHandlerCc(null);
					break;
					
				case "cr": 
					stage.addEventListener(Event.RESIZE, stageResizeHandlerCr);
					stageResizeHandlerCr(null);
					break;
					
				case "bl": 
					stage.addEventListener(Event.RESIZE, stageResizeHandlerBl);
					stageResizeHandlerBl(null);
					break;
					
				case "bc": 
					stage.addEventListener(Event.RESIZE, stageResizeHandlerBc);
					stageResizeHandlerBc(null);
					break;
				
				case "br": 
					stage.addEventListener(Event.RESIZE, stageResizeHandlerBr);
					stageResizeHandlerBr(null);
					break;
			}
		}

		private function stageResizeHandlerTl(event:Event):void
		{
			var cX:int = 0;
			var cY:int = 0;
			
			mContainer.x = cX;
			mContainer.y = cY; 
		}

		private function stageResizeHandlerTc(e:Event):void
		{
			var cX:int = mContainer.stage.stageWidth * .5;
			var cY:int = 								0;
			
			mContainer.x = cX;
			mContainer.y = cY; 
		}
		
		private function stageResizeHandlerTr(event:Event):void
		{
			var cX:int = mContainer.stage.stageWidth;
			var cY:int = 						   0;
			
			mContainer.x = cX;
			mContainer.y = cY; 
		}

		private function stageResizeHandlerCl(event:Event):void
		{
			var cX:int = 								 0;
			var cY:int = mContainer.stage.stageHeight * .5;
			
			mContainer.x = cX;
			mContainer.y = cY; 
		}

		private function stageResizeHandlerCc(event:Event):void
		{
			var cX:int = mContainer.stage.stageWidth  * .5;
			var cY:int = mContainer.stage.stageHeight * .5;
			
			mContainer.x = cX;
			mContainer.y = cY; 
		}

		private function stageResizeHandlerCr(event:Event):void
		{
			var cX:int =	   mContainer.stage.stageWidth;
			var cY:int = mContainer.stage.stageHeight * .5;
			
			mContainer.x = cX;
			mContainer.y = cY; 
		}

		private function stageResizeHandlerBl(event:Event):void
		{
			var cX:int =	  						0;
			var cY:int = mContainer.stage.stageHeight;
			
			mContainer.x = cX;
			mContainer.y = cY; 
		}

		private function stageResizeHandlerBc(event:Event):void
		{
			var cX:int = mContainer.stage.stageWidth  * .5;
			var cY:int = 	  mContainer.stage.stageHeight;
			
			mContainer.x = cX;
			mContainer.y = cY; 
		}

		private function stageResizeHandlerBr(event:Event):void
		{
			var cX:int = mContainer.stage.stageWidth;
			var cY:int = mContainer.stage.stageHeight;
			
			mContainer.x = cX;
			mContainer.y = cY; 
		}


		
		/*
		 * ADD CONTAINER CHILDREN
		 */
	
		private function initContainerChildren():void
		{						
			var cList:Array = mDisplayRegister.containerContainerList();
			var cLength:uint = cList.length;
			
			var container:FixedContainer;
			
			var objectDepth:uint;			
			var placeHolder:Sprite;
			
			for(var j:uint = 0; j < cLength; ++j)
			{
				mFixedDisplay.addChild(new Sprite());	
			}

			for(var k:uint = 0; k < cLength; ++k)
			{
				container   = FixedContainer(cList[k]);
				objectDepth = container.getDepth(); 
				placeHolder = Sprite(mFixedDisplay.getChildAt(objectDepth));
				
				mFixedDisplay.addChildAt(container, objectDepth);
				mFixedDisplay.removeChild(placeHolder);
				
				placeHolder = null;
			}
		}
	}
}
