﻿package de.davidochmann.frameworks.dosite.main.navigation 
{
	/**
	 * @author dochmann
	 */

	import flash.display.Sprite;
	import flash.utils.setTimeout;
	import flash.events.EventDispatcher;
	import de.davidochmann.frameworks.dosite.main.display.Display;
	import de.davidochmann.frameworks.dosite.main.events.DoSiteEvent;
	import de.davidochmann.frameworks.dosite.main.xmlparser.XMLParser;
	import de.davidochmann.frameworks.dosite.main.displayregister.DisplayRegister;
	import de.davidochmann.frameworks.dosite.main.containerbuilder.containerfactory.container.SubsiteContainer;	 
	
	public class Navigation extends EventDispatcher
	{
		private var mDisplayRegister:DisplayRegister;
		private var mXMLParser:XMLParser;
		private var mDisplay:Display;

		private var mCurrent:SubsiteContainer;		
		private var mTransition:Boolean;		
		private var mWait:Boolean;


		
		public function Navigation(pDisplayRegister:DisplayRegister,
								   pXMLParser:XMLParser,
								   pDisplay:Display):void
		{
			setDisplayRegister(pDisplayRegister);
			setXMLParser(pXMLParser);
			setDisplay(pDisplay);
		}

		
		
		/*
		 * PUBLIC FUNCTIONS
		 */
		
		public function open(pURL:String):SubsiteContainer
		{				
			if(mTransition && mWait) return null;
			
			mTransition = false;
			
			var container:Sprite  = mDisplay.subsiteDisplay();
			
			var subsiteList:Array = mDisplayRegister.subsiteContainerList();
			var sLength:uint	  = subsiteList.length;
			
			var subsite:SubsiteContainer;
						
			for(var i:uint = 0; i < sLength; ++i)
			{
				subsite = SubsiteContainer(subsiteList[i]);
				if(subsite.getURL() == pURL) break;
				subsite = null;
			}
		
			if(subsite != null) 
			{
				while(container.numChildren > 0) container.removeChildAt(0);
	
				mCurrent = subsite;
				
				container.addChild(subsite);
				dispatchEvent(new DoSiteEvent(DoSiteEvent.NAVIGATION_SITE_OPEN, subsite));
			}
			
			return subsite;
		}


		public function swap(pURL:String):void
		{
			if(mTransition && mWait || mCurrent.getURL() == pURL) return;
			
			mTransition = true;
			
			var swap:uint = mXMLParser.composition().transition().getSwap();
			dispatchEvent(new DoSiteEvent(DoSiteEvent.NAVIGATION_SITE_SWAP_INIT, current()));
			setTimeout(swapOpen, swap, pURL);
		}

		private function swapOpen(pURL:String):SubsiteContainer
		{	
			mTransition = false;
			
			var container:Sprite  = mDisplay.subsiteDisplay();
			
			var subsiteList:Array = mDisplayRegister.subsiteContainerList();
			var sLength:uint	  = subsiteList.length;
			
			var subsite:SubsiteContainer;
						
			for(var i:uint = 0; i < sLength; ++i)
			{
				subsite = SubsiteContainer(subsiteList[i]);
				if(subsite.getURL() == pURL) break;
				subsite = null;
			}
		
			if(subsite != null) 
			{
				while(container.numChildren > 0) container.removeChildAt(0);
	
				mCurrent = subsite;
				
				container.addChild(subsite);
				dispatchEvent(new DoSiteEvent(DoSiteEvent.NAVIGATION_SITE_SWAP_OPEN, subsite));
			}
			
			return subsite;
		}

		public function transition():Boolean
		{
			return mTransition;
		}

		public function current():SubsiteContainer
		{
			return mCurrent;
		}

		public function initialize():void
		{
			initCurrent();
			initTransition();
			initWait();
			openInitialSubsite();
		}
		
		
		
		/*
		 * GETTER FUNCTIONS
		 */

		public function getDisplayRegister():DisplayRegister
		{
			return mDisplayRegister;
		}

		public function getXMLParser():XMLParser
		{
			return mXMLParser;
		}

		public function getDisplay():Display
		{
			return mDisplay;
		}


		
		/*
		 * SETTER FUNCTIONS
		 */
		 
		public function setDisplayRegister(pDisplayRegister:DisplayRegister):void
		{
			mDisplayRegister = pDisplayRegister;
		}

		public function setXMLParser(pXMLParser:XMLParser):void
		{
			mXMLParser = pXMLParser;
		}

		public function setDisplay(pDisplay:Display):void
		{
			mDisplay = pDisplay;		
		}

		
		
		/*
		 * PRIVATE FUNCTIONS
		 */
		 
		private function initCurrent():void
		{
			mCurrent = SubsiteContainer(mDisplayRegister.subsiteContainerList()[0]);
		}
		
		private function initTransition():void
		{
			mTransition = false;
		}

		private function initWait():void
		{
			mWait = mXMLParser.composition().transition().getWait();
		}

		private function openInitialSubsite():void
		{	
			var init:String = mXMLParser.composition().setup().getInit();
			if(init != "" && init != null) open(init);
		}
	}
}
