﻿package de.davidochmann.frameworks.dosite.main.xmlparser 
{
	/**
	 * @author dochmann
	 */

	import de.davidochmann.frameworks.dosite.main.xmlparser.data.composition.item.Transition;
	import de.davidochmann.frameworks.dosite.main.xmlparser.data.content.Content;
	import de.davidochmann.frameworks.dosite.main.xmlparser.data.content.item.Graphic;
	import de.davidochmann.frameworks.dosite.main.xmlparser.data.composition.item.Setup;
	import de.davidochmann.frameworks.dosite.main.xmlparser.data.content.item.Extensible;
	import de.davidochmann.frameworks.dosite.main.xmlparser.data.composition.Composition;
	import de.davidochmann.frameworks.dosite.main.xmlparser.data.composition.item.layout.Layout;
	import de.davidochmann.frameworks.dosite.main.xmlparser.data.composition.item.layout.item.Subsite;
	import de.davidochmann.frameworks.dosite.main.xmlparser.data.composition.item.layout.item.Fixed;
	 
	public class XMLParser 
	{
		private var mContentXML:XML;
		private var mCompositionXML:XML;
		
		private var mContent:Content;
		private var mComposition:Composition;
		
		
		
		public function XMLParser(pContentXML:XML	  = null,
								  pCompositionXML:XML = null):void
		{
			setContentXML(pContentXML);
			setCompositionXML(pCompositionXML);
		}



		/*
		 * PUBLIC FUNCTIONS
		 */
		
		public function initialize():void
		{	
			initDefaultContent();
			initDefaultComposition();
			initContent();
			initComposition();
		}
		
		public function content():Content
		{
			return mContent;
		}

		public function composition():Composition
		{
			return mComposition;
		}
		
		
		
		/*
		 * GETTER FUNCTIONS
		 */
		
		public function getContentXML():XML
		{
			return mContentXML;
		}

		public function getCompositionXML():XML
		{
			return mCompositionXML;
		}


		
		/*
		 * SETTER FUNCTIONS
		 */
		
		public function setContentXML(pContentXML:XML):void
		{
			mContentXML = pContentXML;
		}
		
		public function setCompositionXML(pCompositionXML:XML):void
		{
			mCompositionXML = pCompositionXML;
		}
		
		
		
		/*
		 * PRIVATE FUNCTIONS
		 */
		
		private function initDefaultContent():void
		{
			mContent = new Content();	
		}

		private function initDefaultComposition():void
		{
			mComposition = new Composition();
			
			var setup:Setup = new Setup();
			
			setup.setInit("");
			setup.setPosition("tc");
			
			
			var transition:Transition = new Transition();

			transition.setSwap(0);
			transition.setWait(false);
			
			
			mComposition.addSetup(setup);
		}

		private function initContent():void
		{			
			if(mContentXML == null) return;
			mContent = new Content();
			
			
			var extensible:Extensible;
			var elength:uint = XMLList(mContentXML.CONTENT.EXTENSIBLE.ITEM).length();
			
			for(var i:uint = 0; i < elength; ++i)
			{
				extensible = new Extensible();
				extensible.setId(String(XMLList(mContentXML.CONTENT.EXTENSIBLE.ITEM[i]).attribute("id")));
				extensible.setPath(String(XMLList(mContentXML.CONTENT.EXTENSIBLE.ITEM[i]).attribute("path")));
				
				mContent.addExtensible(extensible);
			}
			
			
			var graphic:Graphic;
			var gLenght:uint = mContentXML.CONTENT.GRAPHIC.ITEM.length();
			
			for(var j:uint = 0; j < gLenght; ++j)
			{
				graphic = new Graphic();
				graphic.setId(String(XMLList(mContentXML.CONTENT.GRAPHIC.ITEM[j]).attribute("id")));
				graphic.setPath(String(XMLList(mContentXML.CONTENT.GRAPHIC.ITEM[j]).attribute("path")));
								
				mContent.addGraphics(graphic);
			}
		}

		private function initComposition():void
		{
			if(mCompositionXML == null) return;
			mComposition = new Composition();
			
						
			var setup:Setup = new Setup();
			
			setup.setInit(String(XMLList(mCompositionXML.COMPOSITION.SETUP).attribute("init")));
			setup.setPosition(String(XMLList(mCompositionXML.COMPOSITION.SETUP).attribute("position")));

			mComposition.addSetup(setup);


			var transition:Transition = new Transition();
			
			transition.setSwap(uint(XMLList(mCompositionXML.COMPOSITION.TRANSITION).attribute("swap")));
			transition.setWait(Boolean((String(XMLList(mCompositionXML.COMPOSITION.TRANSITION).attribute("wait"))) == "false") ? false : true);
			
			mComposition.addTransition(transition);
												
						
			var layout:Layout = new Layout();
			
			var subsite:Subsite;
			var slength:uint = XMLList(mCompositionXML.COMPOSITION.LAYOUT.SUBSITE.ITEM).length();
			
			for(var i:uint = 0; i < slength; ++i)
			{
				subsite = new Subsite();
				
				subsite.setId(String(XMLList(mCompositionXML.COMPOSITION.LAYOUT.SUBSITE.ITEM[i]).attribute("id")));
				subsite.setURL(String(XMLList(mCompositionXML.COMPOSITION.LAYOUT.SUBSITE.ITEM[i]).attribute("url")));
				subsite.setTitle(String(XMLList(mCompositionXML.COMPOSITION.LAYOUT.SUBSITE.ITEM[i]).attribute("title")));
				subsite.setWidth(uint(XMLList(mCompositionXML.COMPOSITION.LAYOUT.SUBSITE.ITEM[i]).attribute("width")));
				subsite.setHeight(uint(XMLList(mCompositionXML.COMPOSITION.LAYOUT.SUBSITE.ITEM[i]).attribute("height")));
				subsite.setX(int(XMLList(mCompositionXML.COMPOSITION.LAYOUT.SUBSITE.ITEM[i]).attribute("x")));
				subsite.setY(int(XMLList(mCompositionXML.COMPOSITION.LAYOUT.SUBSITE.ITEM[i]).attribute("y")));
				
				layout.addSubsite(subsite);				
			}
			
			var fixed:Fixed;
			var clength:uint = XMLList(mCompositionXML.COMPOSITION.LAYOUT.FIXED.ITEM).length();
			
			for(var j:uint = 0; j < clength; ++j)
			{
				fixed = new Fixed();
				
				fixed.setId(String(XMLList(mCompositionXML.COMPOSITION.LAYOUT.FIXED.ITEM[j]).attribute("id")));
				fixed.setTitle(String(XMLList(mCompositionXML.COMPOSITION.LAYOUT.FIXED.ITEM[j]).attribute("title")));
				fixed.setWidth(uint(XMLList(mCompositionXML.COMPOSITION.LAYOUT.FIXED.ITEM[j]).attribute("width")));
				fixed.setHeight(uint(XMLList(mCompositionXML.COMPOSITION.LAYOUT.FIXED.ITEM[j]).attribute("height")));
				fixed.setX(int(XMLList(mCompositionXML.COMPOSITION.LAYOUT.FIXED.ITEM[j]).attribute("x")));
				fixed.setY(int(XMLList(mCompositionXML.COMPOSITION.LAYOUT.FIXED.ITEM[j]).attribute("y")));
				
				layout.addFixed(fixed);
			}
			
			mComposition.addLayout(layout);
		}
	}
}
