﻿package de.davidochmann.projects.vivaespana.content.scroller 
{
	/**
	 * @author David Ochmann
	 */

	import com.greensock.TweenMax;
	import com.greensock.easing.Strong;
	import flash.events.MouseEvent;
	import flash.events.Event;
	import flash.display.DisplayObject;
	import flash.display.Stage;
	import flash.display.MovieClip;

	[Embed (source="../../../../../../../deploy/swf/content/scroller.swf", symbol="scroller")]

	public class Scroller extends MovieClip 
	{
		public var mBar:MovieClip;
		public var mBackground:MovieClip;
		
		private var mStage:Stage;
		private var mContainer:DisplayObject;
		
		private var mOffset:Number;
		
		
		
		public function Scroller(pStage:Stage, 
								 pContainer:DisplayObject):void
		{
			setStage(pStage);
			setContainer(pContainer);
			
			initStageEvents();
			initMouseEvents();
			checkNeed();
		}
		
		
		
		/*
		 * GETTER FUNCTIONS
		 */
		
		public function getStage():Stage
		{
			return mStage;
		}
		
		public function getContainer():DisplayObject
		{
			return mContainer;
		}
		
		
		
		/*
		 * SETTER FUNCTIONS
		 */
		 
		private function setStage(pStage:Stage):void
		{
			mStage = pStage;
		}
		
		private function setContainer(pContainer:DisplayObject):void
		{
			mContainer = pContainer;
		}
		

		
		/*
		 * PUBLIC FUNCTIONS
		 */
		
		public function checkNeed():void
		{			
			if(mContainer.height <= mStage.stageHeight) 
			{
				move(0);
				removeMouseEvents();
				TweenMax.to(mBar, .5, {alpha:0, ease:Strong.easeOut});
			}
			else
			{
				initMouseEvents();
				TweenMax.to(mBar, .5, {alpha:1, ease:Strong.easeOut});
			}
			
			trace(mBar.alpha, mBar.y);
		}
		
		public function move(pY:int):void
		{			
			if(mContainer.y == pY) return;
			
			var yMin:Number = 0;
			var yMax:Number = mBackground.height - mBar.height;
			
			mBar.y = pY - mOffset;
			
			if(mBar.y <= yMin) mBar.y = yMin;
			if(mBar.y >= yMax) mBar.y = yMax;
			
			var sp:Number = mBar.y / yMax;
			var ny:Number = -sp * (mContainer.height - mStage.stageHeight);
		
			TweenMax.to(mContainer, .5, {y:ny, ease:Strong.easeOut});
		}
		
		
		
		/*
		 * PRIVATE FUNCTIONS
		 */
		
		private function initStageEvents():void
		{
			mStage.addEventListener(Event.RESIZE, stageResizeHandler);
			stageResizeHandler(null);
		}
		
		private function stageResizeHandler(e:Event):void
		{
			checkNeed();
			changePosition();
			resizeBackground();
			checkBarPosition();
		}
		
		private function resizeBackground():void
		{
			mBackground.height = mStage.stageHeight;
		}
		
		private function changePosition():void
		{
			this.x = mStage.stageWidth - width;
		}
		
		private function checkBarPosition():void
		{
			if(mBar.y > mStage.stageHeight - mBar.height) mBar.y = mStage.stageHeight - mBar.height;
			if(mBar.y < 0) mBar.y = 0;
		}
		
		
		private function initMouseEvents():void
		{
			mBar.buttonMode = true;
			
			mBar.addEventListener(MouseEvent.MOUSE_DOWN, barMouseDownHandler);
			mStage.addEventListener(MouseEvent.MOUSE_UP, stageMouseUpHandler);
			mStage.addEventListener(MouseEvent.MOUSE_WHEEL, stageMouseWheelHandler);
			mBackground.addEventListener(MouseEvent.MOUSE_UP, backgroundMouseUpHandler);
		}
		private function removeMouseEvents():void
		{
			mBar.buttonMode = false;
			
			mBar.removeEventListener(MouseEvent.MOUSE_DOWN, barMouseDownHandler);
			mStage.removeEventListener(MouseEvent.MOUSE_UP, stageMouseUpHandler);
			mStage.removeEventListener(MouseEvent.MOUSE_WHEEL, stageMouseWheelHandler);
			mBackground.removeEventListener(MouseEvent.MOUSE_UP, backgroundMouseUpHandler);
		}

		private function barMouseDownHandler(e:MouseEvent):void
		{	
			mOffset = mouseY - mBar.y;
			mStage.addEventListener(MouseEvent.MOUSE_MOVE, stageMouseMoveHandler);
		}

		private function stageMouseUpHandler(e:MouseEvent):void
		{
			mStage.removeEventListener(MouseEvent.MOUSE_MOVE, stageMouseMoveHandler);
		}
		
		private function stageMouseMoveHandler(e:MouseEvent):void
		{
			e.updateAfterEvent();
			move(mouseY);
		}
		
		private function stageMouseWheelHandler(e:MouseEvent):void
		{
			mOffset = 0;
			
			var pos:int = mBar.y - (e.delta * 5);
			move(pos);
		}
		
		
		private function backgroundMouseUpHandler(event:MouseEvent):void
		{
			move(mouseY);
		}
	}
}
